# type: ignore

from unittest.mock import patch, Mock

from gestalt.vault import Vault
from gestalt import merge_into
import pytest
import os
import gestalt
import hvac
from queue import Queue


# Testing member function
def test_merge_into():
    combine1 = {}
    combine2 = {}
    combine3 = {"local": 1234, "pg": {"host": "dict1_pg", "pass": "dict1_pg"}}
    combine4 = {"local": 1234, "pg": {"host": "dict2_pg"}}

    merge_into(combine3, combine1)
    merge_into(combine4, combine1)

    merge_into(combine4, combine2)
    merge_into(combine3, combine2)

    assert combine1 == {
        "local": 1234,
        "pg": {
            "host": "dict2_pg",
            "pass": "dict1_pg"
        }
    }

    assert combine2 == {
        "local": 1234,
        "pg": {
            "host": "dict1_pg",
            "pass": "dict1_pg"
        }
    }


def test_combine_into_empty_dict():
    combine = {}
    merge_into({}, combine)
    assert combine == {}

    combine = {"local": 1234}
    merge_into({}, combine)
    assert combine == {"local": 1234}


# Testing JSON Loading
def test_loading_json():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    x = g.dump()
    assert len(x)


def test_loading_json_file():
    g = gestalt.Gestalt()
    g.add_config_file("./tests/testdata/testjson.json")
    g.build_config()
    x = g.dump()
    assert len(x)


def test_loading_file_dir():
    g = gestalt.Gestalt()
    with pytest.raises(ValueError) as terr:
        g.add_config_file("./tests/testdata")
        assert "is not a file" in terr


def test_loading_file_nonexist():
    g = gestalt.Gestalt()
    with pytest.raises(ValueError) as terr:
        g.add_config_file("./tests/testdata/nothere.yaml")
        g.build_config()
        assert "is not a file" in terr


def test_loading_file_bad_yaml():
    g = gestalt.Gestalt()
    with pytest.raises(ValueError) as terr:
        g.add_config_file("./tests/testdatabad/testyaml.yaml")
        g.build_config()
        assert terr.type is ValueError
        assert "but cannot be read as such" in terr.value.args[0]


def test_loading_file_bad_json():
    g = gestalt.Gestalt()
    with pytest.raises(ValueError) as terr:
        g.add_config_file("./tests/testdatabad/testjson.json")
        g.build_config()
        assert terr.type is ValueError
        assert "but cannot be read as such" in terr.value.args[0]


def test_loading_dir_bad_files():
    g = gestalt.Gestalt()
    with pytest.raises(ValueError) as terr:
        g.add_config_path("./tests/testdatabad")
        g.build_config()
        assert terr.type is ValueError
        assert "but cannot be read as such" in terr.value.args[0]


def test_loading_dir_bad_files_yaml_only():
    g = gestalt.Gestalt()
    with pytest.raises(ValueError) as terr:
        g.add_config_path("./tests/testdatabadyaml")
        g.build_config()
        assert terr.type is ValueError
        assert "but cannot be read as such" in terr.value.args[0]


def test_loading_yaml_file():
    g = gestalt.Gestalt()
    g.add_config_file("./tests/testdata/testyaml.yaml")
    g.build_config()
    x = g.dump()
    assert len(x)


def test_loading_json_nonexist_dir():
    g = gestalt.Gestalt()
    with pytest.raises(ValueError) as terr:
        g.add_config_path("./nonexistpath")
        assert "does not exist" in terr


def test_loading_json_file_not_dir():
    g = gestalt.Gestalt()
    with pytest.raises(ValueError) as terr:
        g.add_config_path("./setup.py")
        assert "is not a directory" in terr


def test_get_wrong_type():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    with pytest.raises(TypeError) as terr:
        g.get_string("numbers")
        assert "is not of type" in terr


def test_get_non_exist_key():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    with pytest.raises(ValueError) as terr:
        g.get_string("non-exist")
        assert "is not in any configuration and no default is provided" in terr


def test_get_key_wrong_type():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    with pytest.raises(TypeError) as terr:
        g.get_string(1234)
        assert "is not of string type" in terr


def test_get_key_wrong_default_type():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    with pytest.raises(TypeError) as terr:
        g.get_string("nonexist", 1234)
        assert "Provided default is of incorrect type" in terr


def test_get_json_string():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    testval = g.get_string("yarn")
    assert testval == "blue skies"


def test_get_json_default_string():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    testval = g.get_string("nonexist", "mydefval")
    assert testval == "mydefval"


def test_get_json_set_default_string():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    g.set_default_string("nonexisttest", "otherdefval")
    testval = g.get_string("nonexisttest")
    assert testval == "otherdefval"


def test_get_json_int():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    testval = g.get_int("numbers")
    assert testval == 12345678


def test_get_json_float():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    testval = g.get_float("strangenumbers")
    assert testval == 123.456


def test_get_json_bool():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    testval = g.get_bool("truthy")
    assert testval is True


def test_get_json_list():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    testval = g.get_list("listing")
    assert testval
    assert "dog" in testval
    assert "cat" in testval


def test_get_json_nested():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    testval = g.get_string("deep.nested1")
    assert testval == "hello"


def test_get_yaml_nested():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    testval = g.get_string("deep_yaml.nest1.nest2.foo")
    assert testval == "hello"


def test_get_yaml_nested_default():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    testval = g.get_string("deep_yaml.nest1.nest2.foo", "default")
    assert testval == "hello"


def test_get_yaml_missing_nested_default():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    testval = g.get_string("deep_yaml.nest1.nest2.fob", "default")
    assert testval == "default"


# Test Set Overriding
def test_set_string():
    g = gestalt.Gestalt()
    g.set_string("mykey", "myval")
    testval = g.get_string("mykey")
    assert testval == "myval"


def test_set_int():
    g = gestalt.Gestalt()
    g.set_int("mykey", 1234)
    testval = g.get_int("mykey")
    assert testval == 1234


def test_set_float():
    g = gestalt.Gestalt()
    g.set_float("mykey", 45.23)
    testval = g.get_float("mykey")
    assert testval == 45.23


def test_set_bool():
    g = gestalt.Gestalt()
    g.set_bool("mykey", False)
    testval = g.get_bool("mykey")
    assert testval is False


def test_set_list():
    g = gestalt.Gestalt()
    g.set_list("mykey", ["hi", "bye"])
    testval = g.get_list("mykey")
    assert testval
    assert "hi" in testval
    assert "bye" in testval


def test_set_int_get_bad():
    g = gestalt.Gestalt()
    g.set_int("mykey", 1234)
    with pytest.raises(TypeError) as terr:
        g.get_string("mykey")
        assert "Given set key is not of type" in terr


def test_set_bad_key_type():
    g = gestalt.Gestalt()
    with pytest.raises(TypeError) as terr:
        g.set_string(1234, "myval")
        assert "is not of string type" in terr


def test_set_bad_type():
    g = gestalt.Gestalt()
    with pytest.raises(TypeError) as terr:
        g.set_string("mykey", 123)
        assert "is not of type" in terr


def test_re_set_bad_type():
    g = gestalt.Gestalt()
    g.set_string("mykey", "123")
    with pytest.raises(TypeError) as terr:
        g.set_int("mykey", 123)
        assert "Overriding key" in terr


def test_set_override():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    testval = g.get_int("numbers")
    assert testval == 12345678
    g.set_int("numbers", 6543)
    testval = g.get_int("numbers")
    assert testval == 6543


def test_set_bad_type_file_config():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    with pytest.raises(TypeError) as terr:
        g.set_string("numbers", "notgood")
        assert "File config has" in terr


def test_set_bad_type_default_config():
    g = gestalt.Gestalt()
    g.set_default_string("mykey", "mystring")
    with pytest.raises(TypeError) as terr:
        g.set_int("mykey", 123)
        assert "Default config has" in terr


# Test Env Variables
def test_get_env_string():
    g = gestalt.Gestalt()
    g.auto_env()
    os.environ["MYKEY"] = "myval"
    testval = g.get_string("mykey")
    assert testval == "myval"


def test_get_env_int():
    g = gestalt.Gestalt()
    g.auto_env()
    os.environ["MYKEY"] = "999"
    testval = g.get_int("mykey")
    assert testval == 999


def test_get_nested_env_string():
    g = gestalt.Gestalt()
    g.auto_env()
    os.environ["MY_KEY"] = "myval"
    testval = g.get_string("my.key")
    assert testval == "myval"


def test_get_env_bad_type():
    g = gestalt.Gestalt()
    g.auto_env()
    os.environ["MY_KEY"] = "myval"
    with pytest.raises(TypeError) as terr:
        g.get_int("my.key")
        assert "could not be converted to type" in terr


# Test Default Values
def test_set_default_string():
    g = gestalt.Gestalt()
    g.set_default_string("mykey", "myval")
    testval = g.get_string("mykey")
    assert testval == "myval"


def test_set_default_int():
    g = gestalt.Gestalt()
    g.set_default_int("mykey", 1234)
    testval = g.get_int("mykey")
    assert testval == 1234


def test_set_default_float():
    g = gestalt.Gestalt()
    g.set_default_float("mykey", 1234.05)
    testval = g.get_float("mykey")
    assert testval == 1234.05


def test_set_default_bool():
    g = gestalt.Gestalt()
    g.set_default_bool("mykey", False)
    testval = g.get_bool("mykey")
    assert testval is False


def test_set_default_list():
    g = gestalt.Gestalt()
    g.set_default_list("mykey", ["bear", "bull"])
    testval = g.get_list("mykey")
    assert testval
    assert "bear" in testval
    assert "bull" in testval


def test_set_default_int_get_bad():
    g = gestalt.Gestalt()
    g.set_default_int("mykey", 1234)
    with pytest.raises(TypeError) as terr:
        g.get_string("mykey")
        assert "Given default set key is not of type" in terr


def test_set_default_string_bad_key():
    g = gestalt.Gestalt()
    with pytest.raises(TypeError) as terr:
        g.set_default_string(1234, "myval")
        assert "Given key is not of string type" in terr


def test_set_default_string_bad_val():
    g = gestalt.Gestalt()
    with pytest.raises(TypeError) as terr:
        g.set_default_string("mykey", 123)
        assert "Input value when setting default" in terr


def test_set_default_string_bad_val_override():
    g = gestalt.Gestalt()
    with pytest.raises(TypeError) as terr:
        g.set_default_string("mykey", "myval")
        g.set_default_int("mykey", 1234)
        assert "Overriding default key" in terr


def test_override_nested_config():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testoverride/")
    g.build_config()
    assert g.get_int("local") == 123456
    assert g.get_string("nested1.nested2") == "final"
    assert g.get_string("pg.host") == "dev_host"
    assert g.get_string("pg.pass") == "def_pass"
    assert g.get_string("nested1.nested3.nested4.deeplevel") == "nested5"


def test_set_default_bad_type_file_config():
    g = gestalt.Gestalt()
    g.add_config_path("./tests/testdata")
    g.build_config()
    with pytest.raises(TypeError) as terr:
        g.set_default_string("numbers", "notgood")
        assert "File config has" in terr


def test_set_default_bad_type_set_config():
    g = gestalt.Gestalt()
    g.set_string("mykey", "mystring")
    with pytest.raises(TypeError) as terr:
        g.set_default_int("mykey", 123)
        assert "Set config has" in terr


def test_vault_setup():
    vault = Vault(role=None, jwt=None)
    assert vault.vault_client.is_authenticated() is True


def test_vault_interpolation(secret_setup):
    g = gestalt.Gestalt()
    g.add_config_file("./tests/testvault/testcorrect.json")
    vault = Vault(role=None, jwt=None)
    g.configure_provider("vault", vault)
    g.build_config()
    secret = g.get_string("test_secret.test_secret")
    assert secret == "test_secret_password"


def test_vault_mount_path(mount_setup):
    g = gestalt.Gestalt()
    g.add_config_file("./tests/testvault/testmount.json")
    g.configure_provider("vault", Vault(role=None, jwt=None))
    g.build_config()
    secret = g.get_string("test_mount.test_mount")
    assert secret == r"test_mount_password\\$"


def test_vault_incorrect_path(mount_setup):
    g = gestalt.Gestalt()
    g.add_config_file("./tests/testvault/testincorrectmount.json")
    g.configure_provider("vault", Vault(role=None, jwt=None))
    with pytest.raises(RuntimeError):
        g.build_config()
        g.get_string("test_mount")


def test_nest_key_for_vault(nested_setup, secret_setup):
    g = gestalt.Gestalt()
    g.add_config_file("./tests/testvault/testnested.json")
    g.configure_provider("vault", Vault(role=None, jwt=None))
    g.build_config()
    secret_db = g.get_string("remoteAPI.database.test_secret")
    secret_slack = g.get_string("remoteAPI.slack.token")
    assert secret_db == "test_secret_password"
    assert secret_slack == "random-token"


def test_read_no_nest_db_role(mock_db_role_request):
    g = gestalt.Gestalt()
    g.add_config_file("./tests/testvault/testsfdynamic.json")
    g.configure_provider("vault", Vault(role=None, jwt=None))
    g.build_config()
    secret_username = g.get_string("username")
    assert secret_username == "foo"


def test_set_vault_key(nested_setup):
    g = gestalt.Gestalt()
    g.configure_provider("vault", Vault(role=None, jwt=None))
    g.set_string(key="test",
                 value="ref+vault://secret/data/testnested#.slack.token")
    g.build_config()
    secret = g.get_string("test")
    assert secret == "ref+vault://secret/data/testnested#.slack.token"


def test_vault_lazy_connect(mock_vault_k8s_auth):
    with patch("gestalt.vault.hvac.Client") as mock_client:
        v = Vault(role="test-role", jwt="test-jwt")
        v.vault_client.auth.token.lookup_self = Mock(
            return_value={
                "data": {
                    "id": "foo",
                    "ttl": "foo",
                    "expire_time": "2024-08-15T22:04:49.82981496Z"
                }
            })
        assert not v._is_connected
        v.get("foo", "foo", ".foo")
        assert v._is_connected
        mock_client().auth.token.lookup_self.assert_called()


def test_vault_start_dynamic_lease(mock_vault_workers):
    mock_response = {
        "lease_id": "1",
        "lease_duration": 5,
        "data": {
            "data": "mock_data"
        },
    }

    mock_vault_client_patch = patch("gestalt.vault.hvac.Client.read",
                                    return_value=mock_response)
    with mock_vault_client_patch as mock_vault_client_read:
        mock_dynamic_token_queue = Mock()
        mock_kube_token = ("kubernetes", "hvs.CAESIEkz-UO8yvfC8v", "2764799")
        with patch(
                "gestalt.vault.Queue",
                side_effect=[mock_dynamic_token_queue],
        ) as mock_queue:
            v = Vault(role=None, jwt=None)
            g = gestalt.Gestalt()
            g.add_config_file("./tests/testvault/testmount.json")
            g.configure_provider("vault", v)
            g.build_config()
            g.get_string("test_mount")

            mock_vault_client_read.assert_called()
            mock_dynamic_token_queue.put_nowait.assert_called()
            assert mock_kube_token == ("kubernetes", "hvs.CAESIEkz-UO8yvfC8v",
                                       "2764799")

            mock_vault_client_read.stop()
            mock_dynamic_token_queue.stop()
            mock_queue.stop()



import os
from datetime import datetime, timedelta, timezone
from queue import Queue
from typing import Any, Dict, List, Optional, Tuple, Union

import hvac  # type: ignore
import requests
from jsonpath_ng import parse  # type: ignore
from requests.exceptions import Timeout
from retry.api import retry_call

from gestalt.provider import Provider
from dateutil.parser import isoparse

EXPIRATION_THRESHOLD_HOURS = 1


class Vault(Provider):
    def __init__(
        self,
        cert: Optional[Tuple[str, str]] = None,
        role: Optional[str] = None,
        jwt: Optional[str] = None,
        url: Optional[str] = os.environ.get("VAULT_ADDR"),
        token: Optional[str] = os.environ.get("VAULT_TOKEN"),
        verify: Optional[bool] = True,
        scheme: str = "ref+vault://",
        delay: int = 60,
        tries: int = 5,
    ) -> None:
        """Initialized vault client and authenticates vault

        Args:
            client_config (HVAC_ClientConfig): initializes vault. URL can be set in VAULT_ADDR
                environment variable, token can be set to VAULT_TOKEN environment variable.
                These will be picked by default if not set to empty string
            auth_config (HVAC_ClientAuthentication): authenticates the initialized vault client
                with role and jwt string from kubernetes
        """
        self._scheme: str = scheme
        self._run_worker = True
        self.dynamic_token_queue: Queue[Tuple[str, str, str]] = Queue()
        self.kubes_token: Optional[Tuple[str, str, str, datetime]] = None

        self._vault_client: Optional[hvac.Client] = None
        self._secret_expiry_times: Dict[str, datetime] = dict()
        self._secret_values: Dict[str, Union[str, int, float, bool,
                                             List[Any]]] = dict()
        self._is_connected: bool = False
        self._role: Optional[str] = role
        self._jwt: Optional[str] = jwt
        self._url: Optional[str] = url
        self._token: Optional[str] = token
        self._cert: Optional[Tuple[str, str]] = cert
        self._verify: Optional[bool] = verify

        self.delay = delay
        self.tries = tries

    @property
    def vault_client(self) -> hvac.Client:
        if self._vault_client is None:
            self._vault_client = hvac.Client(url=self._url,
                                             token=self._token,
                                             cert=self._cert,
                                             verify=self._verify)
        return self._vault_client

    def connect(self) -> None:
        try:
            retry_call(
                self.vault_client.is_authenticated,
                exceptions=(RuntimeError, Timeout),
                delay=self.delay,
                tries=self.tries,
            )
        except requests.exceptions.MissingSchema:
            raise RuntimeError(
                "Gestalt Error: Unable to connect to vault with the given configuration"
            )

        if self._role and self._jwt:
            try:
                hvac.api.auth_methods.Kubernetes(
                    self.vault_client.adapter).login(role=self._role,
                                                     jwt=self._jwt)
                token = retry_call(
                    self.vault_client.auth.token.lookup_self,
                    exceptions=(RuntimeError, Timeout),
                    delay=self.delay,
                    tries=self.tries,
                )

                if token is not None:
                    print("Kubernetes login successful")
                    kubes_token = (
                        "kubernetes",
                        token["data"]["id"],
                        token["data"]["ttl"],
                        token["data"]['expire_time'],
                    )
                    self.kubes_token = kubes_token
            except hvac.exceptions.InvalidPath:
                raise RuntimeError(
                    "Gestalt Error: Kubernetes auth couldn't be performed")
            except requests.exceptions.ConnectionError:
                raise RuntimeError("Gestalt Error: Couldn't connect to Vault")

        self._is_connected = True

    def stop(self) -> None:
        self._run_worker = False

    def __del__(self) -> None:
        self.stop()

    def get(
        self,
        key: str,
        path: str,
        filter: str,
        sep: Optional[str] = "."
    ) -> Union[str, int, float, bool, List[Any]]:
        """Gets secret from vault
        Args:
            key (str): key to get secret from
            path (str): path to secret
            filter (str): filter to apply to secret
            sep (str): delimiter used for flattening
        Returns:
            secret (str): secret
        """
        if not self._is_connected:
            self.connect()
        # if the key has been read before and is not a TTL secret
        if key in self._secret_values and key not in self._secret_expiry_times:
            return self._secret_values[key]

        # if the secret can expire but hasn't expired yet
        if key in self._secret_expiry_times and not self._is_secret_expired(
                key):
            return self._secret_values[key]

        # verify if the token still valid, in case not, call connect()
        self._validate_token_expiration()

        try:
            response = retry_call(
                self.vault_client.read,
                fargs=[path],
                exceptions=(RuntimeError, Timeout),
                delay=self.delay,
                tries=self.tries,
            )
            if response is None:
                raise RuntimeError("Gestalt Error: No secrets found")
            if response["lease_id"]:
                dynamic_token = (
                    "dynamic",
                    response["lease_id"],
                    response["lease_duration"],
                )
                self.dynamic_token_queue.put_nowait(dynamic_token)
            requested_data = response["data"].get("data", response["data"])
        except hvac.exceptions.InvalidPath:
            raise RuntimeError(
                "Gestalt Error: The secret path or mount is set incorrectly")
        except requests.exceptions.ConnectionError:
            raise RuntimeError(
                "Gestalt Error: Gestalt couldn't connect to Vault")
        except Exception as err:
            raise RuntimeError(f"Gestalt Error: {err}")
        if filter is None:
            return requested_data
        secret = requested_data
        jsonpath_expression = parse(f"${filter}")
        match = jsonpath_expression.find(secret)
        if len(match) == 0:
            print("Path returned not matches for your secret")
        returned_value_from_secret = match[0].value
        if returned_value_from_secret == "":
            raise RuntimeError("Gestalt Error: Empty secret!")

        self._secret_values[key] = returned_value_from_secret
        if "ttl" in requested_data:
            self._set_secrets_ttl(requested_data, key)

        # repr is converting the string to RAW string since \\$ was returning $\
        # Then we are removing single quotes (first and last char)
        #
        return str(repr(returned_value_from_secret))[1:-1]

    def _is_secret_expired(self, key: str) -> bool:
        now = datetime.now()
        secret_expires_dt = self._secret_expiry_times[key]
        is_expired = now >= secret_expires_dt
        return is_expired

    def _set_secrets_ttl(self, requested_data: Dict[str, Any],
                         key: str) -> None:
        last_vault_rotation_str = requested_data["last_vault_rotation"].split(
            ".")[0]  # to the nearest second
        last_vault_rotation_dt = datetime.strptime(last_vault_rotation_str,
                                                   "%Y-%m-%dT%H:%M:%S")
        ttl = requested_data["ttl"]
        secret_expires_dt = last_vault_rotation_dt + timedelta(seconds=ttl)
        self._secret_expiry_times[key] = secret_expires_dt

    @property
    def scheme(self) -> str:
        return self._scheme

    def _validate_token_expiration(self) -> None:
        if self.kubes_token is not None:
            expire_time = self.kubes_token[3]
            # Use isoparse to correctly parse the datetime string
            expire_time = isoparse(expire_time)

            # Ensure the parsed time is in UTC
            if expire_time.tzinfo is None:
                expire_time = expire_time.replace(tzinfo=timezone.utc)
            else:
                expire_time = expire_time.astimezone(timezone.utc)

            current_time = datetime.now(timezone.utc)
            # in hours
            delta_time = (expire_time - current_time).total_seconds() / 3600

            if delta_time < EXPIRATION_THRESHOLD_HOURS:
                print(f"Re-authenticating with vault")
                self.connect()
            else:
                print(f"Token still valid for: {delta_time} hours")
        else:
            print(
                f"Can't reconnect, token information: {self.kubes_token}, not valid"
            )



from setuptools import setup, find_packages


def readme():
    with open("README.md") as f:
        return f.read()


with open("requirements.txt") as reqs_file:
    reqs = filter(lambda x: not x.startswith("-"), reqs_file.readlines())
    reqs_list = list(map(lambda x: x.rstrip(), reqs))

setup(
    name="gestalt-cfg",
    version="3.4.2",
    description="A sensible configuration library for Python",
    long_description=readme(),
    long_description_content_type="text/markdown",
    url="https://github.com/clear-street/gestalt",
    author="Clear Street",
    author_email="engineering@clearstreet.io",
    license="MIT",
    packages=find_packages(),
    python_requires=">=3.6",
    install_requires=reqs_list,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development :: Libraries",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
        "Operating System :: OS Independent",
    ],
)
